#! /usr/bin/perl
$RCS_ID = '$Id: gpcon,v 1.54 1994/12/05 12:43:05 hobbs Exp $' ;
$0 =~ s-.*/-- ;
# $CLO = "-s none" ;					# for CLASSIFIED use
$DIR = "." ;		# default dir for plot data files
# $xrl =  1992 ;          # default x range, low
# $xrh =  2100 ;          # default x range, high
$TFN++ ;		# use filename as title
$TFI++ ;		# get title from file info
@DCOLOR_DEF = (		# default mapping of gnuplot color order
    1, 3, 2, 4 .. 64 ) ;
@DCOLOR = @DCOLOR_DEF ; # map actually used
$HelpInfo = <<EOH ;

    A program to control the plotting of a number of files via GNUPLOT.

Usage:  $0  [options]  plot_file  ...
        $0  [-Conv | -Undo]  rdbtable  ...

Options:
    -c N      Plot 2-d curves using column 'N' as the 'Y' values (column 1 is
	      the 'X' values).
    -Conv     Convert the given list of rdbtables to gnuplot files.
    -dir DIR  Use dir 'DIR' instead of the default.
    -file     File output (postscript). Filenames prepended with "ps.".
    -fcolor   File output (color postscript). Filenames prepended with "ps.".
    -gen FILE Generate a script file 'FILE' from interactive inputs.
    -help     Print this help info.
    -in FILE  Commands are read from the script file 'FILE'.
    -lx LABEL Label for the 'x' direction on the plot. Default is the name of
	      column one being used. No embedded spaces.
    -ly LABEL Label for the 'y' direction on the plot. Default is the name of
	      column two being used. No embedded spaces.
    -m [N ...] Multi curve plots (2-D) using columns N ... Default is all
	      columns in data file. This is the default startup condition.
    -nogrid   No grid on plot.
    -notime   No time and date on plot.
    -on PRIN  Use printer 'PRIN' for printing.
    -print    Print output on the B/W printer.
    -pcolor   Print output on the color printer.
    -rx N1 N2 The range to use on the plot in the 'x' direction is N1 to N2.
	      Default is generated automatically.
    -ry N1 N2 The range to use on the plot in the 'y' direction is N1 to N2.
	      Default is generated automatically.
    -s J K L  Show surface plots using coumns J, K, L for X, Y, Z values
    -t        Use the filename as the title (default).
    -Undo     Restore the given list of gnuplot files to rdbtables.

In the first form a number of data files will be plotted from dir '$DIR'. These
files are to be in a format that gnuplot can plot directly.

In the second form the -Com and -Undo options operate on files in the current
dir, and are separate utility operations, no plotting is done.

The plots will have a grid and a time and date stamp by default.

Output is to the screen by default.  Options may be abbreviated.
Uses the program 'gnuplot' and 'gnuplot_x11'.

$RCS_ID
EOH
$MULC++ ;	# default multi plots
$ttyp = 'x11' ;
$ttyp = 'vttek' if $ENV{'TERM'} ne 'xterm' ;		# Mac, at home
$vx = 60 ;	#default x-axis rotation (surface)
$vz = 30 ;	#default z-axis rotation (surface)
$vxc = 5 ;	#default chg for x-axis rotation (surface)
$vzc = 5 ;	#default chg for z-axis rotation (surface)
while( $ARGV[0] =~ /^-/ ){				# Get args
    $_ = shift ;
    if( /-c.*/ ){
	while( $ARGV[0] =~ /^\d+$/ ){
	    push( @PC, shift ) ; }
	next ; }
    if( /-C.*/ ){ $COMO++ ; next ; }
    if( /-d.*/ ){ $DIR = shift ; next ; }
    if( /-f(.*)/ ){
	$NOS++ ;	# (output) not on screen
	$FILEO = 'BW' ;
	if( $1 =~ /^c/ ){ $FILEO = 'COLOR' ; }
	next ; }
    if( /-g.*/ ){
	$SCR++ ;
	$SCRF = shift ;
	open( SF, ">>$SCRF" ) || die "Can't append to script file: $SCRF\n" ;
	next ; }
    if( /-h.*/ ){ print $HelpInfo ; exit 1 ; }
    if( /-i(.*)/ ){
	$CMD++ ;
	$CMDF = shift ;
	open( CF, $CMDF ) || die "Can't open cmd file: $CMDF\n" ;
	next ; }
    if( /-l(.)/ ){
	if( $1 eq 'x' ){
	    $xnam = shift ; print GP "set xlabel \"$xnam\"\n" ; }
	else{
	    $ynam_m = $ynam = shift ; print GP "set ylabel \"$ynam\"\n" ; }
	next ; }
    if( /-m/ ){ $MULC++ ; next ; }
    if( /-no(.)/ ){
	if( $1 eq 'g' ){ $NOGRID++ ; }
	else{ $NOTIME++ ; }
	next ; }
    if( /-o.*/ ){
	$x = shift ;
	$ONPR = "-on $x" ;
	next ; }
    if( /-p(.*)/ ){
	$NOS++ ;	# (output) not on screen
	$PRINTER = 'BW' ;		# B/W printer
	if( $1 =~ /^c/ ){ $PRINTER = 'COLOR' ; } # color printer
	next ; }
    if( /-r(.)/ ){
	if( $1 eq 'x' ){ $xrl = shift ; $xrh = shift ; }
	else{ $yrl = shift ; $yrh = shift ; }
	next ; }
    if( /-s.*/ ){
	$SURF++ ;
	$NOGRID++ ;
	for (1..3){ push( @PC, shift ) ; }
	next ; }
    if( /-t.*/ ){ $TFN++ ; next ; }
    if( /-x.*/ ){ $xbug++ ; next ; }
    if( /-U.*/ ){ $UNDO++ ; next ; }
    die "\nBad arg: $_, for help type: \"$0 -help\"\n" ;
}
@PC = ( 2 ) unless @PC ;	# default plot column (2-d)
if( $COMO ){ &do_como ; exit ; }
if( $UNDO ){ &do_undo ; exit ; }
$CDIR = $ENV{'PWD'} ;		# curr dir
chdir( $DIR ) || die "Can't cd to: $DIR\n" ;
open( GP, "| gnuplot" ) || die "Can't open pipe\n" ;
select( GP ) ; $|++ ; select( STDOUT ) ; $|++ ;

@F = @ARGV ;				# list of files to plot
&get_col_names ; # get column names from a data file
print GP "set term $ttyp\n" ;
print GP "set data style lines\n" ;
print GP "set xrange [ $xrl : $xrh ]\n" if $xrh ;
print GP "set yrange [ $yrl : $yrh ]\n" if $yrh ;
print GP "set nokey\n" ;
print GP "set time\n" unless $NOTIME ;
print GP "set grid\n" unless $NOGRID ;
if( $SURF ){						# surface plots
    print GP "set xlabel \"$col_nam[$PC[0]]\"\n" ;
    print GP "set ylabel \"$col_nam[$PC[1]]\"\n" ;
    print GP "set zlabel \"$col_nam[$PC[2]]\"\n" ;
    print GP "set param; set autoscale\n" ; }
else{							# 2-d plots
    &set_xnam unless $xnam ;	# default x label
    &set_ynam unless $ynam ; } 	# default y label
if( $MULC ){	# multi col plots
    @PC = () ;
    for( $x=2 ; $x < @col_nam ; $x++ ){
	push( @PC, $x ) ; }
    @PC = ( 2 ) unless @PC ;
    print GP "set key\n" ;
    &set_ynam ;
}
if( $FILEO ){
    if( $FILEO eq 'BW' ){
	print GP "set term postscript\n" ; }
    else{
	print GP "set term postscript color\n" ; } }
if( $PRINTER ){
    if( $PRINTER eq 'BW' ){
	print GP "set term postscript\n" ; }
    else{
	print GP "set term postscript color\n" ; }
    print GP "set output \"| printout $ONPR $CLO -type raw\"\n" ; }

$CAN++ if $CMD && $NOS ;	# special mode: cmd file, not to screen
for( $iok=$i=1; $i <= @F ; ){			# file loop
    $_ = $F[$i-1] ;
    if( -z ){
	print "$i .. $_ Warning, file has ZERO SIZE, Can't plot\n" ;
	$i++ ;
	next ; }
    $t = $_ ;
    unless( $TFN ){		# title is filename
	if( /^[^.]+\.([^.]+)\.([^.]+)$/ ){	# 3 part (or more) name
	    $fclas = $1 ;
	    $fmod = $2 ;
	    $t = "$fclas $tit2 $fmod" ; } }
    $t = &get_tit_ffi ;		# info from file
    if( $skipfile ){
	$i++ ;
	$i = 1 if $i + $AUTOAD > @F && @F > 1 ;
	next; }
    print GP "set title \"$t\"\n" ;
    &set_pstg ;			# set print string '$p_stg' for gnuplot
    &set_xnam ;
    &set_ynam ;
    if( $NOS && ! $CMD ){		# ouptut not on screen
	print GP "set output \"$CDIR/ps.$_\"\n" if $FILEO ;
	print GP "$p_stg\n" ;
	$i++ ; }
    else{
	if( $iok ){			# input char ok
	    print $p_stg if $xbug ;
	    # $_ = <STDIN> ; #<<<<<<<<<<<<<<< wait, at home <<<<<<<<<<<<<
	    unless( $NOS ){
		print GP "$p_stg" ;
		&add_one( $i -1 + $AUTOAD ) if $AUTOAD ; }
	    unless( $inited++ ){
		print "\nAt any time Type 'q' to quit, 'h' for help.\n" 
		    unless $NOS ;
		print "$i .. $_ : " unless $CMD ; }
	    else{
		unless( $NOS ){
		    print "$i .. $_ : " ; } } 	# online prompt
	}
	else{
	    print "What ? ... For help enter 'h' <RET> " ; }
	&action ; }
}
sub add_one { # (index)				# add a plot to current plot
    $_ = $F[$_[0]] ;
    # print "AO: $_[0] ... $_\n" ; #<<<<<<<<<<
    $add++ ;
    $keyt = $2 ; # need this ???
    &set_pstg ;
    print GP "set key\n" ;
    print GP $p_stg unless $NOS ;
    print $p_stg if $xbug ;
    print "Added $1 : " unless $NOS || $AUTOAD ;
}
sub action {				# get info back from STDIN or CF
    while( 1 ){
	$iok = 0 ;
	if( $CMD ){	# input from file
	    $r = <CF> ;
	    exit unless $r ;
	    chop( $r ) ; }
	else{		# normal input, fm STDIN
	    chop( $r = <STDIN> ) ;
	    if( $SCR ){						# script file
		if( &need_w ){
		    print SF "w\n" if $notfst++ ; }
		print SF $r, "\n" unless $r =~ /^q/ ;	} }
	if( $r =~ /^aa\s*(\d*)/ ){	# auto add mode 	'aa'
	    if( $1 ){
		$AUTOAD = $1 ; }
	    else{
		$AUTOAD = 0 ; }
	    next ; }
	if( $r =~ /^a\s+(\d+)\s*(.*)$/ ){# add other plot file to curr plot
	    $add++ ;			# add another plot	'a'
	    &add_one( $1 -1 ) ;
	    next ; }
	elsif( $r =~ /^c\s+(\d+)/ ){	# use col N 		'c'
	    $MULC = 0 ;
	    $PC_rdy++ ;
	    print GP "set nokey\n" ;
	    if( $SURF ){
		$SURF = 0 ;
		print GP "set nopar\n" ; }
	    @PC = split( ' ', $r ) ;
	    shift @PC ;
	    &set_xnam ;
	    &set_ynam ;
	    next ; }
	elsif( $r =~ /^cm\s/ ){	# chg color mapping (1,2,3,...)	'cm'
	    @cm = split( ' ', $r ) ;
	    shift @cm ;
	    if( $cm[0] eq 'r' ){		# reset to default
		@DCOLOR = @DCOLOR_DEF ; }
	    else{				# set order
		for( $x=0 ; $x < @cm ; $x++ ){
		    $DCOLOR[$x] = $cm[$x] ; } }
	    next ; }
	elsif( $r =~ /^h/ ){	# help info  			'h'
	    &sh_help ;
	    next ; }
	elsif( $r =~ /^f/ ){	# gen PS file			'f'
	    next if $CMD ;
	    $color = "" ;
	    $color = "color" if $r =~ /^fc/ ;
	    print GP "set term postscript $color\n" ;
	    $outnr++ ;
	    print GP "set output \"$CDIR/ps.$outnr\"\n" ;
	    print GP "replot\n" ;
	    print GP "set term $ttyp\n" ;
	    print GP "set output\n" ;
	    print GP "replot\n" ;
	    print "PS file: ps.$outnr : " ;
	    next ; }
	elsif( $r eq 'l' ){		# list of plot files 	'l'
	    unless( $wrows ){
		`stty size 0<&2` =~ /^(\d+)/ ;
		$wrows = $1 -1 ; }
	    for( $k=$j=1; $j <= @F ; $j++, $k++ ){
		$x = $F[$j-1] ;
		print "    $j .. $x\n" ;
		if( $k >= $wrows ){
		    $k = 0 ;
		    print "Press <RET> " ;
		    last if <STDIN> !~ /^\n$/ ; }
	    }
	    print "$i .. $_ : " ;
	    next ; }
	elsif( $r =~ /^lx\s+(.*)$/ ){	# x label change	'lx'
	    print GP "set xlabel \"$1\"\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^ly\s+(.*)$/ ){	# y label change	'ly'
	    print GP "set ylabel \"$1\"\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^m/ ){		# use multi col [N ...]	'm'
	    $MULC++ ;
	    if( $SURF ){
		$SURF = 0 ;
		print GP "set nopar\n" ; }
	    @tx = split( ' ', $r ) ;
	    shift @tx ;
	    if( @tx ){
		$PC_rdy++ ;
		@PC = @tx ; }
	    else{
		$PC_rdy = 0 ;
		@PC = () ;
		for( $x=2 ; $x < @col_nam ; $x++ ){
		    push( @PC, $x ) ; }
		@PC = ( 2 ) unless @PC ; }
	    print GP "set key\n" ;
	    &set_xnam ;
	    &set_ynam ;
	    next ; }
	elsif( $r =~ /^on?\s+(\S+)/ ){	# on PRINTER		'o'
	    $ONPR = "-on $1" ;
	    next ; }
	elsif( $r =~ /^p/ ){		# print curr plot	'p,pc'
	    next if $CMD ;
	    $color = "" ;
	    $color = "color" if $r =~ /^pc/ ;
	    print GP "set term postscript $color\n" ;
	    print GP "set output \"| printout $ONPR $CLO -type raw\"\n" ;
	    print GP "replot\n" ;
	    print GP "set term $ttyp\n" ;
	    print GP "set output\n" ;
	    print GP "replot\n" ;
	    print "Printed: $_ : " ;
	    next ; }
	elsif( $r =~ /^rx\s+([-\d.]+)\s+([-\d.]+)$/ ){ # x range change 'rx'
	    print GP "set xrange [ $1 : $2 ]\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^rx\s*$/ ){	# x autoscale		'rx'
	    print GP "set autoscale x\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^ry\s+([-\d.]+)\s+([-\d.]+)$/ ){# y range change  'ry'
	    print GP "set yrange [ $1 : $2 ]\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^ry\s*$/ ){	# y autoscale		'ry'
	    print GP "set autoscale y\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^s\s+(\d+)/ ){	# surface plot		's'
	    unless( $SURF ){
		$SURF++ ;
		print GP "set param\; set autoscale; set nogrid\n" ; }
	    @PC = split( ' ', $r ) ;
	    shift @PC ;
	    print GP "set xlabel \"$col_nam[$PC[0]]\"\n" ;
	    print GP "set ylabel \"$col_nam[$PC[1]]\"\n" ;
	    print GP "set zlabel \"$col_nam[$PC[2]]\"\n" ;
	    next ; }
	elsif( $r =~ /^sh\S*\s+(\S+)/ ){# show column names	'sh'
	    if( $1 =~ /^g/ ){
		&gnu_cmds ; }
	    elsif( $1 =~ /^cm/ ){ # color mapping
		print "@DCOLOR[0 .. 15]\n" ;
		print "1=Red 2=Green 3=Blue 4=Violet 5=LtBlue\n" ;
	    }
	    else{
		print "\n@col_nam\n" ; }
	    next ; }
	elsif( $r =~ /^t\s+(.*)$/ ){	# title change		't'
	    print GP "set title \"$1\"\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^vv\s*(-?\d*)/ ){	# view [N] z-axis chg	'vv'
	    $vzc = $1 if $1 ;
	    $vz += $vzc ;
	    print GP "set view ,$vz\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^v\s*(-?\d*)/ ){	# view [N] x-axis chg	'v'
	    $vxc = $1 if $1 ;
	    $vx += $vxc ;
	    print GP "set view $vx\n" ;
	    print GP "replot\n" unless $NOS ;
	    next ; }
	elsif( $r =~ /^w\s*(\d*)$/ ){	# wait [file]		'w'
	    last unless $CMD ;
	    if( $NOS ){
		if( $FILEO ){
		    if( $1 ){
			print GP "set output \"$CDIR/ps.$1\"\n" ; }
		    else{
			$outnr++ ;
			print GP "set output \"$CDIR/ps.$outnr\"\n" ; } }
	    }
	    else{		# wait for <RET>
		$_ = <STDIN> ;
		if( /^g/ ){	# start input cmd file over
		    seek( CF, 0, 0 ) ; }
		exit if /^q/ ; }
	    next ; }
	elsif( $r =~ /^ .*/ ){		# gnuplot cmd		' '
	    print GP "$r\n" ;
	    next ; }
	if( $add ){
	    $add = 0 ;
	    print GP "set nokey\n" unless $MULC ; }
	last ; } # end while loop
    if( $r eq "'" ){			# previous plot		"'"
	$iok++ ;
	$x = $i ;
	$i = $pre_i ;
	$pre_i = $x ; }
    else{
	$pre_i = $i ; }
    if( $r eq '' ){			# foreward 1 plot	''
	$i++ ;
	$iok++ ; }
    elsif( $r eq 'b' ){			# backward 1 plot	'b'
	$i-- ;
	$iok++ ; }
    elsif( $r =~ /^(\d+)\s*(.*)$/ ){	# show plot nr N	'N'
	$iok++ ;
	$keyt = "" ;
	$keyt = $2 if $2 ;
	$i = $1 ; }
    $iok++ if $r eq '.' ;		# plot same file	'.'
    exit if $r eq 'q' ;			# quit			'q'
    $i = @F if $i < 1 ;			# check in range
    $i = 1 if $i + $AUTOAD > @F ;
}
sub set_xnam {		# set $xnam, 2-d plot
    if( $MULC || @PC == 1 ){
	$xnam = $col_nam[1] ; }
    else{
	$xnam = $col_nam[$PC[0]] ; }
    print GP "set xlabel \"$xnam\"\n" ;
}
sub set_ynam {		# set $ynam, 2-d plot
    if( $MULC ){
	$ynam = $ynam_m if $ynam_m; }
    else{
	if( @PC == 1 ){
	    $ynam = $col_nam[$PC[0]] ; }
	else{
	    $ynam = $col_nam[$PC[1]] ;
	    for( $c = 2 ; $c < @PC ; $c++ ){
		$ynam .= " $col_nam[$PC[$c]]" ; } }
    }
    print GP "set ylabel \"$ynam\"\n" ;
}
sub need_w {		# return 1 if a 'wait' is needed (gen script file)
    return 1 if $r eq '' || $r =~ /^\d/ || $r eq 'b' ||
	$r eq "'" || $r eq '.' || $r =~ /^\s/ || $r eq 'q' ;
    0 ;
}
sub set_pstg {			# set print string '$p_stg' for gnuplot
    $kt_str = "" ;
    $kt_str = "ti \"$keyt\"" if $keyt ;
    $verb = 'plot' ;
    $verb = 'splot' if $SURF ;
    $verb = 'replot' if $add && ! $NOS ;
    $verb = '' if $add && $NOS ;
    $dc = 0 unless $add ;		# index into @DCOLOR
    if( $SURF ){			# surface plot
	$p_stg = "$verb \"$_\" us $PC[0]:$PC[1]:$PC[2]" ; }
    elsif( $MULC ){			# multi column 2-d plot
	$p_stg = "$verb" ;
	for( $x = "", $c = 0 ; $c < @PC ; $c++ ){
	    $p_stg .= "$x \"$_\" us $PC[$c] ti \"$col_nam[$PC[$c]]\"" .
		" $DCOLOR[$dc++]" ;
	    $x = ',' ; } }
    else{				# single column 2-d plot
	if( @PC == 1 ){
	    $p_stg = "$verb \"$_\" us $PC[0]" ; }
	else{
	    $p_stg = "$verb" ;
	    for( $x = "", $c = 1 ; $c < @PC ; $c++ ){
		$p_stg .= "$x \"$_\" us $PC[0]:$PC[$c]" ;
		$x = ',' ; } }
    }
    $p_stg .= " $kt_str\n" ;
    if( $CAN ){
	chop( $xx = $p_stg ) ;
	unless( $add ){
	    $p_stg2 = $xx ; }
	else{
	    $p_stg2 .= ", $xx" ; } }
}
sub get_tit_ffi {			# return title from file ($_) info
    local( $in ) ;
    $skipfile = 0 ;	# flag to skip curr file plotting
    open( TMP, $_ ) || die "\nCan't open $_\n" ;
    $title = "" ;
    while( $in = <TMP> ){
	if( $in =~ /^#ti\s+(.*)/ ){	# title for plots $title = $1 ;
	    $title = $1 ; }
	elsif( $in =~ /^#ly\s+(.*)/ ){	# Y label for multi col plots
	    $ynam_m = $1 ; }
	elsif( $in =~ /^## / ){		# col names
	    @col_nam = split( ' ', $in ) ;
	    unless( $PC_rdy ){
		@PC = () ;
		for( $x=2 ; $x < @col_nam ; $x++ ){
		    push( @PC, $x ) ; }
		@PC = ( 2 ) unless @PC ;
	    }
	    last ; }
	elsif( $in =~ /^#gp\s+(.*)/ ){	# gnuplot cmd ... every plot
	    print GP "$1\n" ; }
	elsif( $in =~ /^#gp_setup/ ){	# setup file only, no plotting.
	    $skipfile++ ; }
    }
    close TMP ;
    $title = $_ unless $title ;
    $title ;
}
sub get_col_names {	# get column names from a data file, sto in @col_nam
    $_ = $F[0] ;
    &get_tit_ffi ;
    warn "\nBad format plot file: $F[0]\n" unless @col_nam || $skipfile ;
}
sub do_como {		# convert the given list of rdbtables to gnuplot files
    print STDERR "Converting ... " ;
    for $file (@ARGV){
	print STDERR "$file " ;
	($fnew = $file) =~ s/\.[^.]+$/.gp/ ;
	die "\nAlready gnuplot file: $file\n" if $fnew eq $file ;
	open( FI, $file ) || die "Can't open file: $file\n" ;
	open( FO, ">$fnew" ) || die "Can't open output file: $fnew\n" ;
	while( <FI> ){
	    if( /^\s*#/ ){	# skip header documentation
		print FO ;
		next ; }
	    s/^/## / ; print FO ;	# comment out header
	    $_ = <FI> ;
	    s/^/## / ; print FO ;
	    last ; }
	while( <FI> ){			# chg empty lines to <NL>
	    if( /^\s*$/ ){
		print FO "\n" ; }
	    else{
		print FO $_ ; } }
	unlink $file ;
    }
    print STDERR "\n" ;
}
sub do_undo {		# restore the given list of gnuplot files to rdbtables
    print STDERR "Restoring ... " ;
    fileloop: for $file (@ARGV){
	print STDERR "$file " ;
	($fnew = $file) =~ s/\.[^.]+$/.rdb/ ;
	die "\nAlready rdbtable: $file\n" if $fnew eq $file ;
	open( FI, $file ) || die "Can't open file: $file\n" ;
	open( FO, ">$fnew" ) || die "Can't open output file: $fnew\n" ;
	@t = () ;
	while( <FI> ){			# get commented lines
	    if( /^\s*#/ ){
		push( @t, $_ ) ;
		next ; }
	    last ; }
	if( @t < 2 ){
	    print STDERR "(INVALID-FORM) " ; }
	else{
	    for( $i=$#t ; $i >= 0 ; $i-- ){	# uncomment last 2
		$t[$i] =~ s/\s*#+\s*// ;
		last if $i < $#t ; } }
	print FO @t ;
	print FO $_ ;
	$nr_tabs = $t[$#t] =~ tr/\t/\t/ ;
	while( <FI> ){			# chg empty lines to tbl form
	    if( /^\s*$/ ){
		print FO "\t" x $nr_tabs, "\n" ; }
	    else{
		print FO $_ ; } }
	unlink $file ;
    }
    print STDERR "\n" ;
}
sub sh_help {					# show online help info
    print <<EOH ;

Enter    Result .... (After entering a request press <RET>)

l        Show a list of numbers and names of plot files
N [xxx]  Plot file number N, legend will be "xxx" if other file added
(null)   Plot next file in order
b        Back up one file and plot
'        Plot previous file 
c N      Ready for 2-D plot using column N
cm N ... Change color mapping ('r' to reset to default)
m [N ...] Ready for multi curve plots (2-D) using cols N ... (def. is all cols)
on PRIN  Setup to use printer 'PRIN'
p        print current plot, 'pc' for color printer
f        Generate a postscript file (prefix 'ps.'), 'fc' for color postscript
a N      Add file number N to current plot
aa [N]   Auto Add mode; N is the delta; no N turns mode off.
t xxx    Change title to "xxx"
lx xxx   Change 'X' label to "xxx"
ly xxx   Change 'Y' label to "xxx"
rx N1 N2 Change 'X' range to N1 thru N2, just 'rx' for autoscale
ry N1 N2 Change 'Y' range to N1 thru N2, just 'ry' for autoscale
s J K L  Ready for surface plot using coumns J, K, L for X, Y, Z values
v  [N]   View change (surface) around x-axis of (+/-)N degrees, N new default
vv [N]   View change (surface) around z-axis of (+/-)N degrees, N new default
 cmd     Gnuplot command (must start with a space)
.	 Plot same file
sh X     X=c: Show column names, X=cm: color mapping, X=g: gnu commands
q        Quit
EOH
}
sub gnu_cmds {					# show online gnu cmd help
    print <<EOH ;
set data style lines (or points linespoints impulses steps boxes)
set xrange [ 10 : 20 ]
set xlabel "name"
set nokey
set time
set grid
set autoscale
set title "title"
show xrange (or other of above)
replot
EOH
}
# In command file only:

# w	 Wait [FILE] (print, gen file 'FILE')
# g	 Go to start of cmd file (ignored if not screen output)

